using Godot;
using System;

public class Overlays : Node
{
    [Export]
    private KeyList _cycleOverlayKey;

    [Export]
    private Resource[] _overlayResources;
    private Overlay[] _overlays;
    private int _overlayIdx = 0;
    private Overlay Current => _overlays[_overlayIdx];

    [Export]
    private Resource _tileGridResource;
    private TileGrid _tileGrid;

    [Export]
    private NodePath _worldGridPath;
    private WorldGrid _worldGrid;

    public override void _Ready()
    {
        _tileGrid = (TileGrid)_tileGridResource;
        _overlays = new Overlay[_overlayResources.Length];
        for (int i = 0; i < _overlayResources.Length; i++)
        {
            _overlays[i] = (Overlay)_overlayResources[i];
        }
        _worldGrid = GetNode<WorldGrid>(_worldGridPath);
    }

    public override void _Input(InputEvent @event)
    {
        base._Input(@event);

        if (!(@event is InputEventKey keyEvent))
            return;

        if (!(keyEvent.Pressed))
            return;

        if ((KeyList)keyEvent.Scancode != _cycleOverlayKey)
            return;

        CycleOverlay();
    }

    public override void _Process(float delta)
    {
        base._Process(delta);

        for (int i = 0; i < _tileGrid.Count; i++)
        {
            var tile = _tileGrid[i];
            var material = _worldGrid.GetTileMaterial(i);

            Current.Process(tile, material);
        }
    }

    private void CycleOverlay()
    {
        _overlayIdx++;
        _overlayIdx %= _overlays.Length;

        for (int i = 0; i < _tileGrid.Count; i++)
        {
            var tile = _tileGrid[i];
            var material = _worldGrid.GetTileMaterial(i);

            Current.Initialise(tile, material);
        }
    }
}
